#!/usr/bin/python3

import sys
import re
import subprocess
from  pyparsing import Keyword, QuotedString, Suppress, Regex, OneOrMore
from pyparsing import Forward, Group, Dict, delimitedList
from  pyparsing import lineno

class Cfg:

    def __init__(self):
        self.cfg = {}
        def cfg(s):
            m = re.match('([^=]+)(?:="(.*)")?', s)
            return m.group(1), m.group(2) or True
        self.cfg = dict([ cfg(l) for l
                         in subprocess.check_output(
                             [ 'rustc', '--print=cfg'],
                             encoding='utf8').splitlines() ])
        pass

    def compare(self, key, value):
        if key in self.cfg:
            return value == self.cfg[key]
        return False

    pass

cfg = Cfg()

"""
https://doc.rust-lang.org/reference/conditional-compilation.html

ConfigurationPredicate :
     ConfigurationOption
   | ConfigurationAll
   | ConfigurationAny
   | ConfigurationNot

ConfigurationOption :
   IDENTIFIER (= (STRING_LITERAL | RAW_STRING_LITERAL))?

ConfigurationAll
   all ( ConfigurationPredicateList? )

ConfigurationAny
   any ( ConfigurationPredicateList? )

ConfigurationNot
   not ( ConfigurationPredicate )

ConfigurationPredicateList
   ConfigurationPredicate (, ConfigurationPredicate)* ,?
"""

_predicate_ = Forward()

_all_ = (
    Suppress(Keyword('all')) +
    Suppress('(') +
    delimitedList(_predicate_) +
    Suppress(')')
).setParseAction(lambda predicates: set([True]) == set(predicates))
    
_any_ = (
    Suppress(Keyword('any')) +
    Suppress('(') +
    delimitedList(_predicate_) +
    Suppress(')')
).setParseAction(lambda predicates: True in set(predicates))

_not_ = (
    Suppress(Keyword('not')) +
    Suppress('(') +
    _predicate_ +
    Suppress(')')
).setParseAction(lambda predicate: not predicate[0])

_option_ = (
    (
        Regex('[A-Za-z_]+') +
        Suppress('=') +
        QuotedString('"')
    ).setParseAction(lambda i_s: cfg.compare(i_s[0], i_s[1])) |
    (
        Regex('[A-Za-z_]+')
    ).setParseAction(lambda i: cfg.compare(i[0], True))
)

_predicate_ <<= (
    _all_ |
    _any_ |
    _not_ |
    _option_
)

_cfg_ = (
    Suppress(Keyword('cfg')) +
    Suppress('(') +
    _predicate_ +
    Suppress(')')
)

def evaluate_cfg(s):
    result = _cfg_.parseString(s.replace('\\', ''))
    return result[0]
    
def filter_dependencies(f):
    result = []
    suppress = False
    for l in f.split('\n'):
        if m := re.match(r'^\[target\.(.)(cfg\(.*)\1\.dependencies\.', l):
            cfg = m.group(2).replace('\\', '')
            suppress = not evaluate_cfg(cfg)
            pass
        elif l.startswith('['):
            suppress = False
            pass
        if not suppress:
            result.append(l)
            pass
        else:
            print("# Suppress", l, file=sys.stderr)
            pass
        pass
    return '\n'.join(result)+'\n'
    pass

if __name__ == '__main__':
    for p in sys.argv[1:]:
        old = open(p).read()
        new = filter_dependencies(old)
        if old != new:
            with open(p, 'w') as f:
                f.write(new)
                pass
            pass
        pass
    pass
